@@ -395,7 +395,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
         {
           flags|=XValue;
           if (((flags & XNegative) != 0) && (x != (ssize_t *) NULL))
-            *x=(-*x);
+            *x=CastDoubleToLong(-1.0**x);
         }
     }
   if ((*p == '+') || (*p == '-'))
@@ -416,7 +416,7 @@ MagickExport MagickStatusType GetGeometry(const char *geometry,ssize_t *x,
         {
           flags|=YValue;
           if (((flags & YNegative) != 0) && (y != (ssize_t *) NULL))
-            *y=(-*y);
+            *y=CastDoubleToLong(-1.0**y);
         }
     }
   if ((flags & PercentValue) != 0)
@@ -1597,7 +1597,7 @@ MagickExport MagickStatusType ParseMetaGeometry(const char *geometry,ssize_t *x,
       *width=CastDoubleToUnsigned(MagickMax(floor(scale_factor*stasis_width+
         0.5),1.0));
       *height=CastDoubleToUnsigned(MagickMax(floor(scale_factor*stasis_height+
-        0.5),.10));
+        0.5),1.0));
     }
   if ((flags & GreaterValue) != 0)
     {
