@@ -1935,8 +1935,9 @@ static MagickBooleanType ValidateXMPProfile(Image *image,
 }
 #else
 static MagickBooleanType ValidateXMPProfile(Image *image,
-  const StringInfo *profile,ExceptionInfo *exception)
+  const StringInfo *magick_unused(profile),ExceptionInfo *exception)
 {
+  magick_unreferenced(profile);
   (void) ThrowMagickException(exception,GetMagickModule(),
     MissingDelegateWarning,"DelegateLibrarySupportNotBuiltIn","'%s' (XML)",
     image->filename);
@@ -2005,11 +2006,10 @@ MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  SyncImageProfiles() synchronizes image properties with the image profiles.
-%  Currently we only support updating the EXIF resolution and orientation.
 %
 %  The format of the SyncImageProfiles method is:
 %
-%      MagickBooleanType SyncImageProfiles(Image *image)
+%      void SyncImageProfiles(Image *image)
 %
 %  A description of each parameter follows:
 %
@@ -2156,7 +2156,7 @@ static void WriteProfileShort(const EndianType endian,
   (void) memcpy(p,buffer,2);
 }
 
-static MagickBooleanType SyncExifProfile(const Image *image,unsigned char *exif,
+static void SyncExifProfile(const Image *image,unsigned char *exif,
   size_t length)
 {
 #define MaxDirectoryStack  16
@@ -2175,7 +2175,7 @@ static MagickBooleanType SyncExifProfile(const Image *image,unsigned char *exif,
   } DirectoryInfo;
 
   DirectoryInfo
-    directory_stack[MaxDirectoryStack] = {{ 0 }};
+    directory_stack[MaxDirectoryStack] = { { 0, 0 } };
 
   EndianType
     endian;
@@ -2199,7 +2199,7 @@ static MagickBooleanType SyncExifProfile(const Image *image,unsigned char *exif,
     *directory;
 
   if (length < 16)
-    return(MagickFalse);
+    return;
   id=(ssize_t) ReadProfileShort(LSBEndian,exif);
   if ((id != 0x4949) && (id != 0x4D4D))
     {
@@ -2220,7 +2220,7 @@ static MagickBooleanType SyncExifProfile(const Image *image,unsigned char *exif,
         break;
       }
       if (length < 16)
-        return(MagickFalse);
+        return;
       id=(ssize_t) ReadProfileShort(LSBEndian,exif);
     }
   endian=LSBEndian;
@@ -2230,15 +2230,15 @@ static MagickBooleanType SyncExifProfile(const Image *image,unsigned char *exif,
     if (id == 0x4D4D)
       endian=MSBEndian;
     else
-      return(MagickFalse);
+      return;
   if (ReadProfileShort(endian,exif+2) != 0x002a)
-    return(MagickFalse);
+    return;
   /*
     This the offset to the first IFD.
   */
   offset=(ssize_t) ReadProfileLong(endian,exif+4);
   if ((offset < 0) || ((size_t) offset >= length))
-    return(MagickFalse);
+    return;
   directory=exif+offset;
   level=0;
   entry=0;
@@ -2373,10 +2373,10 @@ static MagickBooleanType SyncExifProfile(const Image *image,unsigned char *exif,
     }
   } while (level > 0);
   exif_resources=DestroySplayTree(exif_resources);
-  return(MagickTrue);
+  return;
 }
 
-static MagickBooleanType Sync8BimProfile(const Image *image,
+static void Sync8BimProfile(const Image *image,
   const StringInfo *profile)
 {
   size_t
@@ -2404,18 +2404,18 @@ static MagickBooleanType Sync8BimProfile(const Image *image,
     if (ReadProfileByte(&p,&length) != 0x4D)
       continue;
     if (length < 7)
-      return(MagickFalse);
+      return;
     id=ReadProfileMSBShort(&p,&length);
     count=(ssize_t) ReadProfileByte(&p,&length);
     if ((count >= (ssize_t) length) || (count < 0))
-      return(MagickFalse);
+      return;
     p+=count;
     length-=count;
     if ((*p & 0x01) == 0)
       (void) ReadProfileByte(&p,&length);
     count=(ssize_t) ReadProfileMSBLong(&p,&length);
     if ((count > (ssize_t) length) || (count < 0))
-      return(MagickFalse);
+      return;
     if ((id == 0x3ED) && (count == 16))
       {
         if (image->units == PixelsPerCentimeterResolution)
@@ -2434,32 +2434,151 @@ static MagickBooleanType Sync8BimProfile(const Image *image,
         WriteProfileShort(MSBEndian,(unsigned short) image->units,p+12);
       }
     if (id == 0x0422)
-      (void) SyncExifProfile(image,p,count);
+      SyncExifProfile(image,p,count);
     p+=count;
     length-=count;
   }
+  return;
+}
+
+static void ReplaceXmpValue(StringInfo *profile,size_t start,size_t end,
+  const char *value)
+{
+  char
+    *datum;
+
+  size_t
+    length,
+    new_length,
+    value_length;
+
+  length=GetStringInfoLength(profile);
+  value_length=strlen(value);
+  new_length=length-(end-start)+value_length;
+  if (new_length > length)
+    SetStringInfoLength(profile,new_length);
+  datum=(char *) GetStringInfoDatum(profile);
+  memmove(datum+start+value_length,datum+end,length-end);
+  memcpy(datum+start,value,value_length);
+  if (new_length < length)
+    {
+      SetStringInfoLength(profile,new_length);
+      *(datum+new_length)='\0';
+    }
+}
+
+static MagickBooleanType GetXmpOffsets(const StringInfo *profile,
+  const char *tag,size_t *start,size_t *end)
+{
+  char
+    *datum,
+    *pos;
+
+  size_t
+    length,
+    tag_length;
+
+  datum=(char *) GetStringInfoDatum(profile);
+  length=GetStringInfoLength(profile);
+  pos=strstr(datum,tag);
+  tag_length=strlen(tag);
+  if ((pos == (char *) NULL) || ((pos-datum) < 1) || (*(pos-1) != '<') ||
+      (((pos-datum)+tag_length) > length) || (*(pos+tag_length) != '>'))
+    return(MagickFalse);
+  *start=(pos-datum)+tag_length+1;
+  pos=strstr(datum+*start,"<");
+  if (pos == (char *) NULL)
+    return(MagickFalse);
+  *end=(pos-datum);
   return(MagickTrue);
 }
 
-MagickPrivate MagickBooleanType SyncImageProfiles(Image *image)
+static void GetXmpNumeratorAndDenominator(double value,size_t *numerator,
+  size_t *denominator)
 {
-  MagickBooleanType
-    status;
+  double
+    df;
 
+  *numerator=0;
+  *denominator=1;
+  if (value <= MagickEpsilon)
+    return;
+  *numerator=1;
+  df=1.0;
+  while(fabs(df - value) > MagickEpsilon)
+  {
+    if (df < value)
+      (*numerator)++;
+    else
+      {
+        (*denominator)++;
+        *numerator=(size_t) (value*(*denominator));
+      }
+    df=*numerator/(double)*denominator;
+  }
+}
+
+static void SyncXmpProfile(const Image *image,StringInfo *profile)
+{
+  char
+    value[MagickPathExtent];
+
+  size_t
+    denominator,
+    end,
+    numerator,
+    start;
+
+  *value='\0';
+  if (GetXmpOffsets(profile,"tiff:XResolution",&start,&end) != MagickFalse)
+    {
+      GetXmpNumeratorAndDenominator(image->resolution.x,&numerator,
+        &denominator);
+      (void) FormatLocaleString(value,MagickPathExtent,"%zu/%zu",numerator,
+        denominator);
+      ReplaceXmpValue(profile,start,end,value);
+    }
+  if (GetXmpOffsets(profile,"tiff:YResolution",&start,&end) != MagickFalse)
+    {
+      if ((fabs(image->resolution.x-image->resolution.y) > MagickEpsilon) ||
+          (*value == '\0'))
+        {
+          GetXmpNumeratorAndDenominator(image->resolution.y,&numerator,
+            &denominator);
+          (void) FormatLocaleString(value,MagickPathExtent,"%zu/%zu",numerator,
+            denominator);
+        }
+      ReplaceXmpValue(profile,start,end,value);
+    }
+  if (GetXmpOffsets(profile,"tiff:ResolutionUnit",&start,&end) != MagickFalse)
+    {
+      (void) FormatLocaleString(value,MagickPathExtent,"%d",
+        ((int) image->units)+1);
+      ReplaceXmpValue(profile,start,end,value);
+    }
+  if (GetXmpOffsets(profile,"tiff:Orientation",&start,&end) != MagickFalse)
+    {
+      (void) FormatLocaleString(value,MagickPathExtent,"%d",
+        (int) image->orientation);
+      ReplaceXmpValue(profile,start,end,value);
+    }
+}
+
+MagickPrivate void SyncImageProfiles(Image *image)
+{
   StringInfo
     *profile;
 
-  status=MagickTrue;
   profile=(StringInfo *) GetImageProfile(image,"8BIM");
   if (profile != (StringInfo *) NULL)
-    if (Sync8BimProfile(image,profile) == MagickFalse)
-      status=MagickFalse;
+    Sync8BimProfile(image,profile);
   profile=(StringInfo *) GetImageProfile(image,"EXIF");
   if (profile != (StringInfo *) NULL)
-    if (SyncExifProfile(image,GetStringInfoDatum(profile),
-      GetStringInfoLength(profile)) == MagickFalse)
-      status=MagickFalse;
-  return(status);
+    SyncExifProfile(image,GetStringInfoDatum(profile),GetStringInfoLength(
+      profile));
+  profile=(StringInfo *) GetImageProfile(image,"XMP");
+  if (profile != (StringInfo *) NULL)
+    SyncXmpProfile(image,profile);
 }
 
 static void UpdateClipPath(unsigned char *blob,size_t length,
