@@ -17,7 +17,7 @@
 %                                 July 1999                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -106,15 +106,6 @@ static MagickBooleanType
 %
 */
 
-static MagickBooleanType IsAVI(const unsigned char *magick,const size_t length)
-{
-  if (length < 4)
-    return(MagickFalse);
-  if (memcmp(magick,"RIFF",4) == 0)
-    return(MagickTrue);
-  return(MagickFalse);
-}
-
 static MagickBooleanType IsPNG(const unsigned char *magick,const size_t length)
 {
   if (length < 8)
@@ -161,6 +152,7 @@ static MagickBooleanType IsVIDEO(const unsigned char *magick,
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
 static const char *GetIntermediateFormat(const ImageInfo *image_info)
 {
   const char
@@ -198,11 +190,11 @@ static Image *ReadVIDEOImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -266,6 +258,8 @@ static Image *ReadVIDEOImage(const ImageInfo *image_info,
         intermediate_format);
       (void) ConcatenateMagickString(options,command,MagickPathExtent);
       AcquireUniqueFilename(read_info->unique);
+      (void) AcquireUniqueSymbolicLink(image_info->filename,
+        read_info->filename);
       (void) FormatLocaleString(command,MagickPathExtent,
         GetDelegateCommands(delegate_info),read_info->filename,options,
         read_info->unique);
@@ -278,9 +272,11 @@ static Image *ReadVIDEOImage(const ImageInfo *image_info,
         command,message,exception);
       if (exit_code == 0)
         images=ReadImage(read_info,exception);
-      else if (*message != '\0')
-        (void) ThrowMagickException(exception,GetMagickModule(),
-          DelegateError,"VideoDelegateFailed","`%s'",message);
+      else
+        if (*message != '\0')
+          (void) ThrowMagickException(exception,GetMagickModule(),DelegateError,
+            "VideoDelegateFailed","`%s'",message);
+      (void) RelinquishUniqueFileResource(read_info->filename);
       (void) RelinquishUniqueFileResource(read_info->unique);
       if (images != (Image *) NULL)
         for (next=images; next != (Image *) NULL; next=next->next)
@@ -341,7 +337,6 @@ ModuleExport size_t RegisterVIDEOImage(void)
   (void) RegisterMagickInfo(entry);
   entry=AcquireMagickInfo("VIDEO","AVI","Microsoft Audio/Visual Interleaved");
   entry->decoder=(DecodeImageHandler *) ReadVIDEOImage;
-  entry->magick=(IsImageFormatHandler *) IsAVI;
   entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
   entry=AcquireMagickInfo("VIDEO","FLV","Flash Video Stream");
@@ -558,7 +553,7 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
     delay;
 
   Image
-    *coalesce_image;
+    *clone_images;
 
   ImageInfo
     *write_info;
@@ -590,20 +585,20 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   /*
     Write intermediate files.
   */
-  coalesce_image=CoalesceImages(image,exception);
-  if (coalesce_image == (Image *) NULL)
+  clone_images=CloneImageList(image,exception);
+  if (clone_images == (Image *) NULL)
     return(MagickFalse);
   file=AcquireUniqueFileResource(basename);
   if (file != -1)
     file=close(file)-1;
-  (void) FormatLocaleString(coalesce_image->filename,MagickPathExtent,"%s",
+  (void) FormatLocaleString(clone_images->filename,MagickPathExtent,"%s",
     basename);
   count=0;
   write_info=CloneImageInfo(image_info);
@@ -611,7 +606,7 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
   *write_info->magick='\0';
   status=MagickTrue;
   intermediate_format=GetIntermediateFormat(image_info);
-  for (p=coalesce_image; p != (Image *) NULL; p=GetNextImageInList(p))
+  for (p=clone_images; p != (Image *) NULL; p=GetNextImageInList(p))
   {
     char
       previous_image[MagickPathExtent];
@@ -649,6 +644,7 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
         {
           blob=(unsigned char *) FileToBlob(previous_image,~0UL,&length,
             exception);
+          magick_fallthrough;
         }
         default:
         {
@@ -700,7 +696,7 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
 
       options=AcquireString("");
       (void) FormatLocaleString(options,MagickPathExtent,"-plays %i",
-        (int) coalesce_image->iterations);
+        (int) clone_images->iterations);
       option=GetImageOption(image_info,"video:pixel-format");
       if (option != (const char *) NULL)
         {
@@ -737,7 +733,7 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
     Relinquish resources.
   */
   count=0;
-  for (p=coalesce_image; p != (Image *) NULL; p=GetNextImageInList(p))
+  for (p=clone_images; p != (Image *) NULL; p=GetNextImageInList(p))
   {
     delay=100.0*p->delay/MagickMax(1.0*p->ticks_per_second,1.0);
     for (i=0; i < (ssize_t) MagickMax((1.0*delay+1.0)/3.0,1.0); i++)
@@ -749,6 +745,6 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
     (void) CopyMagickString(p->filename,image_info->filename,MagickPathExtent);
   }
   (void) RelinquishUniqueFileResource(basename);
-  coalesce_image=DestroyImageList(coalesce_image);
+  clone_images=DestroyImageList(clone_images);
   return(status);
 }
